﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2017 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.GeneralizedQuadraticAssignment.Algorithms.LAHC {
  public sealed class LAHCContext : SingleSolutionContext<ISingleObjectiveSolutionScope<GQAPSolution>> {
    [Storable]
    private IValueParameter<GQAP> problem;
    public GQAP Problem {
      get { return problem.Value; }
      set { problem.Value = value; }
    }

    [Storable]
    private IValueParameter<GQAPSolution> bestSolution;
    public GQAPSolution BestSolution {
      get { return bestSolution.Value; }
      set { bestSolution.Value = value; }
    }

    [Storable]
    private IValueParameter<DoubleArray> memory;
    public DoubleArray Memory {
      get { return memory.Value; }
      set { memory.Value = value; }
    }

    [Storable]
    private IValueParameter<IntValue> lastSuccess;
    public int LastSuccess {
      get { return lastSuccess.Value.Value; }
      set { lastSuccess.Value.Value = value; }
    }
    
    [StorableConstructor]
    private LAHCContext(bool deserializing) : base(deserializing) { }
    private LAHCContext(LAHCContext original, Cloner cloner)
    : base(original, cloner) {
      problem = cloner.Clone(original.problem);
      bestSolution = cloner.Clone(original.bestSolution);
      memory = cloner.Clone(original.memory);
      lastSuccess = cloner.Clone(original.lastSuccess);
    }
    public LAHCContext() {
      Parameters.Add(problem = new ValueParameter<GQAP>("Problem"));
      Parameters.Add(bestSolution = new ValueParameter<GQAPSolution>("BestFoundSolution"));
      Parameters.Add(memory = new ValueParameter<DoubleArray>("Memory"));
      Parameters.Add(lastSuccess = new ValueParameter<IntValue>("LastSuccess"));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new LAHCContext(this, cloner);
    }

    public ISingleObjectiveSolutionScope<GQAPSolution> ToScope(GQAPSolution code, double fitness = double.NaN) {
      var name = Problem.Encoding.Name;
      var scope = new SingleObjectiveSolutionScope<GQAPSolution>(code,
        name + "Solution", fitness, Problem.Evaluator.QualityParameter.ActualName) {
        Parent = Scope
      };
      scope.Variables.Add(new Variable(name, code.Assignment));
      scope.Variables.Add(new Variable("Evaluation", code.Evaluation));
      return scope;
    }
  }
}